package javawrapper;

/**
 * Manages mortality behaviors and data.
 * <p>Copyright: Copyright (c) 2003 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>Cleared history for release 6.09 (LEM)
 * <br>February 4, 2008: Updated to conform to Java 6 compliance (LEM)
 * <br>February 13, 2008: Updated dead flag to integer (LEM)
 */

public class MortalityBehaviors
    extends BehaviorTypeBase {

  /**Mortality at zero growth for each species*/
  protected ModelVector mp_fMortAtZeroGrowth = new ModelVector(
      "Mortality at Zero Growth",
      "mo_mortAtZeroGrowth", "mo_mazgVal",
      0, ModelVector.FLOAT);

  /**Light-dependent mortality for each species*/
  protected ModelVector mp_fLightDependentMortality = new ModelVector(
      "Light-Dependent Mortality",
      "mo_lightDependentMortality",
      "mo_ldmVal", 0,
      ModelVector.FLOAT);

  /**Senescence mortality alpha for each species*/
  protected ModelVector mp_fRandomMortalityAlpha = new ModelVector(
      "Senescence Mortality Alpha",
      "mo_randomMortAlpha",
      "mo_rmaVal", 0,
      ModelVector.FLOAT);

  /**Senescence mortality beta for each species*/
  protected ModelVector mp_fRandomMortalityBeta = new ModelVector(
      "Senescence Mortality Beta",
      "mo_randomMortBeta", "mo_rmbVal",
      0, ModelVector.FLOAT);

  /**Random adult mortality for each species*/
  protected ModelVector mp_fAdultRandomMortality = new ModelVector(
      "Adult Background Mortality Rate", "mo_nonSizeDepMort", "mo_nsdmVal", 0,
      ModelVector.FLOAT);

  /**Random juvenile mortality for each species*/
  protected ModelVector mp_fJuvenileRandomMortality = new ModelVector(
      "Juvenile Background Mortality Rate", "mo_randomJuvenileMortality",
      "mo_rjmVal", 0, ModelVector.FLOAT);

  /**DBH at onset of senescence for each species*/
  protected ModelVector mp_fDbhAtOnsetOfSenescence = new ModelVector(
      "DBH at Onset of Senescence, in cm", "mo_dbhAtOnsetOfSenescence",
      "mo_daoosVal", 0, ModelVector.FLOAT);

  /**Juvenile self-thinning slope for each species*/
  protected ModelVector mp_fJuvenileSelfThinningSlope = new ModelVector(
      "Juvenile Self-Thinning Slope", "mo_juvSelfThinSlope", "mo_jstsVal", 0,
      ModelVector.FLOAT);

  /**Juvenile self-thinning intercept for each species*/
  protected ModelVector mp_fJuvenileSelfThinningIntercept = new ModelVector(
      "Juvenile Self-Thinning Intercept", "mo_juvSelfThinIntercept",
      "mo_jstiVal", 0, ModelVector.FLOAT);

  /**Adult self-thinning slope for each species*/
  protected ModelVector mp_fAdultSelfThinningSlope = new ModelVector(
      "Adult Self-Thinning Slope", "mo_adultSelfThinSlope",
      "mo_astsVal", 0, ModelVector.FLOAT);

  /**Adult self-thinning intercept for each species*/
  protected ModelVector mp_fAdultSelfThinningIntercept = new ModelVector(
      "Adult Self-Thinning Intercept", "mo_adultSelfThinIntercept",
      "mo_astiVal", 0, ModelVector.FLOAT);

  /**Adult self-thinning DBH for each species*/
  protected ModelVector mp_fAdultSelfThinningMaxDbh = new ModelVector(
      "Maximum DBH for Adult Self-Thinning", "mo_adultSelfThinMaxDbh",
      "mo_astmdVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 1 "a" parameter*/
  protected ModelVector mp_fSnag1WeibullA = new ModelVector(
      "Weibull Annual \"a\" Parameter for Snag Size Class 1 Mortality",
      "mo_snag1WeibullA", "mo_s1waVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 1 "a" parameter*/
  protected ModelVector mp_fSnag2WeibullA = new ModelVector(
      "Weibull Annual \"a\" Parameter for Snag Size Class 2 Mortality",
      "mo_snag2WeibullA", "mo_s2waVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 3 "a" parameter*/
  protected ModelVector mp_fSnag3WeibullA = new ModelVector(
      "Weibull Annual \"a\" Parameter for Snag Size Class 3 Mortality",
      "mo_snag3WeibullA", "mo_s3waVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 1 "b" parameter*/
  protected ModelVector mp_fSnag1WeibullB = new ModelVector(
      "Weibull Annual \"b\" Parameter for Snag Size Class 1 Mortality",
      "mo_snag1WeibullB", "mo_s1wbVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 2 "b" parameter*/
  protected ModelVector mp_fSnag2WeibullB = new ModelVector(
      "Weibull Annual \"b\" Parameter for Snag Size Class 2 Mortality",
      "mo_snag2WeibullB", "mo_s2wbVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 3 "b" parameter*/
  protected ModelVector mp_fSnag3WeibullB = new ModelVector(
      "Weibull Annual \"b\" Parameter for Snag Size Class 3 Mortality",
      "mo_snag3WeibullB", "mo_s3wbVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 1 upper DBH value*/
  protected ModelVector mp_fSnagSizeClass1Dbh = new ModelVector(
      "Weibull Upper DBH of Snag Size Class 1",
      "mo_snagSizeClass1DBH", "mo_sc1dVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 2 upper DBH value*/
  protected ModelVector mp_fSnagSizeClass2Dbh = new ModelVector(
      "Weibull Upper DBH of Snag Size Class 2",
      "mo_snagSizeClass2DBH", "mo_sc2dVal", 0, ModelVector.FLOAT);

  /**Weibull snag mortality - snag size class 1 upper DBH value
   * @deprecated This value now has a species-specific version and this one
   * will eventually be eliminated.*/
  protected ModelFloat m_fSnagSizeClass1DbhDep = new ModelFloat(0,
      "Upper DBH of Snag Size Class 1", "mo_snagSizeClass1");

  /**Weibull snag mortality - snag size class 2 upper DBH value
   * @deprecated This value now has a species-specific version and this one
   * will eventually be eliminated.*/
  protected ModelFloat m_fSnagSizeClass2DbhDep = new ModelFloat(0,
      "Upper DBH of Snag Size Class 2", "mo_snagSizeClass2");

  /**DBH at asymptotic maximum mortality - for senescence*/
  protected ModelInt m_iDbhAtAsymptoticMaximumMortality = new ModelInt(0,
      "DBH of Maximum Senescence Mortality Rate, as an integer in cm",
      "mo_dbhAtAsympMaxMort");

  /**NCI Mortality - Maximum crowding radius*/
  protected ModelVector mp_fNCIMaxCrowdingRadius = new ModelVector(
      "NCI Max Radius of Crowding Neighbors, in m",
      "mo_prnciMaxCrowdingRadius", "mo_pmcrVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Neighbor DBH effect (alpha) parameter*/
  protected ModelVector mp_fNCINeighDBHEffect = new ModelVector(
      "NCI Neighbor DBH Effect (alpha)", "mo_prnciNeighDBHEff",
      "mo_pndeVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Neighbor distance effect (beta) parameter*/
  protected ModelVector mp_fNCINeighDistanceEffect = new ModelVector(
      "NCI Neighbor Distance Effect (beta)", "mo_prnciNeighDistEff",
      "mo_pndseVal", 0, ModelVector.FLOAT);

  /**NCI minimum DBH for crowding neighbors, for each species; all species
   * required*/
  protected ModelVector mp_fNCIMinNeighborDBH = new ModelVector(
      "NCI Minimum Neighbor DBH, in cm", "mo_nciMinNeighborDBH",
      "mo_nmndVal", 0, ModelVector.FLOAT, true);

  /**NCI Mortality - Shading coefficient (m)*/
  protected ModelVector mp_fNCIShadingEffectCoefficient = new ModelVector(
      "NCI Shading Effect Coefficient (m)", "mo_nciShadingCoefficient",
      "mo_nscVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Shading exponent (n)*/
  protected ModelVector mp_fNCIShadingEffectExponent = new ModelVector(
      "NCI Shading Effect Exponent (n)", "mo_nciShadingExponent",
      "mo_nseVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Size sensitivity to NCI parameter (gamma)*/
  protected ModelVector mp_fNCISizeSensToNCI = new ModelVector(
      "NCI Size Sensitivity to NCI (gamma)", "mo_prnciSizeSensNCI",
      "mo_pssnVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Maximum probability of survival*/
  protected ModelVector mp_fNCIMaxProbSurvival = new ModelVector(
      "NCI Max Survival Probability (0-1)", "mo_prnciMaxPotentialSurvival",
      "mo_pmpsVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Size effect mode (X0)*/
  protected ModelVector mp_fNCISizeEffectMode = new ModelVector(
      "NCI Size Effect Mode, in cm", "mo_prnciSizeEffectMode",
      "mo_psemVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Size effect variance (Xb)*/
  protected ModelVector mp_fNCISizeEffectVariance = new ModelVector(
      "NCI Size Effect Variance, in cm", "mo_prnciSizeEffectVar",
      "mo_psevVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Crowding Effect Slope (C)*/
  protected ModelVector mp_fNCISlope = new ModelVector(
      "NCI Crowding Effect Slope (C)", "mo_prnciNCISlope", "mo_pnslVal", 0,
      ModelVector.FLOAT);

  /**NCI Mortality - Crowding Effect Steepness (D)*/
  protected ModelVector mp_fNCISteepness = new ModelVector(
      "NCI Crowding Effect Steepness (D)", "mo_prnciNCISteepness", "mo_pnstVal",
      0, ModelVector.FLOAT);

  /**NCI Mortality - Storm effect - medium damage*/
  protected ModelVector mp_fNCIStormEffectMed = new ModelVector(
      "NCI Storm Effect - Medium Damage (0-1)", "mo_prnciStormEffMedDmg",
      "mo_psemdVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Storm effect - full damage*/
  protected ModelVector mp_fNCIStormEffectFull = new ModelVector(
      "NCI Storm Effect - Complete Damage (0-1)",
      "mo_prnciStormEffFullDmg", "mo_psefdVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Neighbor storm effect - medium damage*/
  protected ModelVector mp_fNCINeighStormEffMed = new ModelVector(
      "NCI Neighbor Storm Damage (eta) - Medium (0-1)",
      "mo_prnciNeighStormEffMedDmg", "mo_pnsemdVal", 0, ModelVector.FLOAT);

  /**NCI Mortality - Neighbor storm effect - full damage*/
  protected ModelVector mp_fNCINeighStormEffFull = new ModelVector(
      "NCI Neighbor Storm Damage (eta) - Complete (0-1)",
      "mo_prnciNeighStormEffFullDmg", "mo_pnsefdVal", 0, ModelVector.FLOAT);

  /**Resource mortality - scaling factor (rho)*/
  protected ModelVector mp_fResMortScalingFactor = new ModelVector(
      "Growth/Resource - Scaling Factor (rho)",
      "mo_resMortScalingFactor", "mo_rmsfVal", 0, ModelVector.FLOAT);

  /**Resource mortality - function mode (mu)*/
  protected ModelVector mp_fResMortFunctionMode = new ModelVector(
      "Growth/Resource - Function Mode (mu)",
      "mo_resMortMode", "mo_rmmVal", 0, ModelVector.FLOAT);

  /**Resource mortality - growth increase in survival (delta)*/
  protected ModelVector mp_fResMortGrowthIncSurv = new ModelVector(
      "Growth/Resource - Survival Increase with Growth (delta)",
      "mo_resMortGrowthIncSurv", "mo_rmgisVal", 0, ModelVector.FLOAT);

  /**Resource mortality - low growth function shape (sigma)*/
  protected ModelVector mp_fResMortLoGrowthShape = new ModelVector(
      "Growth/Resource - Low-Growth Survival Parameter (sigma)",
      "mo_resMortLoGrowthShape", "mo_rmlgsVal", 0, ModelVector.FLOAT);

  /**Competition mortality - Shape parameter (Z)*/
  protected ModelVector mp_fCompMortShape = new ModelVector(
      "Competition Mortality Shape Parameter (Z)",
      "mo_CompMort", "mo_cmVal", 0, ModelVector.FLOAT);

  /**Competition mortality - maximum parameter (max)*/
  protected ModelVector mp_fCompMortMax = new ModelVector(
      "Competition Mortality Maximum Parameter (max)",
      "mo_CompMortMax", "mo_cmmVal", 0, ModelVector.FLOAT);

  /**Density self-thinning mortality asymptote (A)*/
  protected ModelVector mp_fDensSelfThinAsymptote = new ModelVector(
      "Density Self-Thinning Asymptote (A)",
      "mo_selfThinAsymptote", "mo_staVal", 0, ModelVector.FLOAT);

  /**Density self-thinning mortality diameter effect (C)*/
  protected ModelVector mp_fDensSelfThinDiamEffect = new ModelVector(
      "Density Self-Thinning Diameter Effect (C)",
      "mo_selfThinDiamEffect", "mo_stdieVal", 0, ModelVector.FLOAT);

  /**Density self-thinning mortality density effect (S)*/
  protected ModelVector mp_fDensSelfThinDensityEffect = new ModelVector(
      "Density Self-Thinning Density Effect (S)",
      "mo_selfThinDensityEffect", "mo_stdeeVal", 0, ModelVector.FLOAT);

  /**Density self-thinning neighborhood radius*/
  protected ModelVector mp_fDensSelfThinNeighRadius = new ModelVector(
      "Density Self-Thinning Neighborhood Radius, in m",
      "mo_selfThinRadius", "mo_strVal", 0, ModelVector.FLOAT);

  /**Density self-thinning minimum density for mortality*/
  protected ModelVector mp_fDensSelfThinMinDensity = new ModelVector(
      "Density Self-Thinning Minimum Density for Mortality (#/ha)",
      "mo_minDensityForMort", "mo_mdfmVal", 0, ModelVector.FLOAT);

  /**Logistic bi-level mortality - low-light "a"*/
  protected ModelVector mp_fLogBiLevLoLiteA = new ModelVector(
      "Logistic Bi-Level - Low-Light \"a\"",
      "mo_logBilevLoLiteA", "mo_lbllaVal", 0, ModelVector.FLOAT);

  /**Logistic bi-level mortality - low-light "b"*/
  protected ModelVector mp_fLogBiLevLoLiteB = new ModelVector(
      "Logistic Bi-Level - Low-Light \"b\"",
      "mo_logBilevLoLiteB", "mo_lbllbVal", 0, ModelVector.FLOAT);

  /**Logistic bi-level mortality - high-light "a"*/
  protected ModelVector mp_fLogBiLevHiLiteA = new ModelVector(
      "Logistic Bi-Level - High-Light \"a\"",
      "mo_logBilevHiLiteA", "mo_lbhlaVal", 0, ModelVector.FLOAT);

  /**Logistic bi-level mortality - high-light "b"*/
  protected ModelVector mp_fLogBiLevHiLiteB = new ModelVector(
      "Logistic Bi-Level - High-Light \"b\"",
      "mo_logBilevHiLiteB", "mo_lbhlbVal", 0, ModelVector.FLOAT);

  /** Logistic bi-level mortality - threshold for high-light mortality*/
  protected ModelVector mp_fLogBiLevHiLiteThreshold = new ModelVector(
      "Logistic Bi-Level - High-Light Mortality Threshold (0-100)",
      "mo_logBilevHiLiteThreshold", "mo_lbhltVal", 0, ModelVector.FLOAT);

  /**Stochastic bi-level mortality - low-light probability of mortality*/
  protected ModelVector mp_fStochBiLevLoLiteMortProb = new ModelVector(
     "Stochastic Bi-Level - Low-Light Mortality Probability (0-1)",
     "mo_stochBilevLoLiteMortProb", "mo_sbllmpVal", 0, ModelVector.FLOAT);

  /**Stochastic bi-level mortality - high-light probability of mortality*/
  protected ModelVector mp_fStochBiLevHiLiteMortProb = new ModelVector(
     "Stochastic Bi-Level - High-Light Mortality Probability (0-1)",
     "mo_stochBilevHiLiteMortProb", "mo_sbhlmpVal", 0, ModelVector.FLOAT);

  /** Stochastic bi-level mortality - threshold for high-light mortality*/
  protected ModelVector mp_fStochBiLevHiLiteThreshold = new ModelVector(
      "Stochastic Bi-Level - High-Light Mortality Threshold",
      "mo_stochBilevHiLiteThreshold", "mo_sbhltVal", 0, ModelVector.FLOAT);

  /** Height-GLI weibull mortality - a */
  protected ModelVector mp_fHeightGLIWeibA = new ModelVector(
      "Height-GLI Weibull - a", "mo_heightGLIWeibA", "mo_hgwaVal", 0,
      ModelVector.FLOAT);

  /** Height-GLI weibull mortality - b */
  protected ModelVector mp_fHeightGLIWeibB = new ModelVector(
      "Height-GLI Weibull - b", "mo_heightGLIWeibB", "mo_hgwbVal", 0,
      ModelVector.FLOAT);

  /** Height-GLI weibull mortality - c */
  protected ModelVector mp_fHeightGLIWeibC = new ModelVector(
      "Height-GLI Weibull - c", "mo_heightGLIWeibC", "mo_hgwcVal", 0,
      ModelVector.FLOAT);

  /** Height-GLI weibull mortality - d */
  protected ModelVector mp_fHeightGLIWeibD = new ModelVector(
      "Height-GLI Weibull - d", "mo_heightGLIWeibD", "mo_hgwdVal", 0,
      ModelVector.FLOAT);

  /** Height-GLI weibull mortality - max mortality */
  protected ModelVector mp_fHeightGLIWeibMaxMort = new ModelVector(
      "Height-GLI Weibull - Max Mortality (0 - 1)", "mo_heightGLIWeibMaxMort",
      "mo_hgwmmVal", 0, ModelVector.FLOAT);

  /** Height-GLI weibull mortality - browsed a */
  protected ModelVector mp_fHeightGLIWeibBrowsedA = new ModelVector(
      "Height-GLI Weibull - Browsed a", "mo_heightGLIWeibBrowsedA",
      "mo_hgwbaVal", 0, ModelVector.FLOAT);

  /** Height-GLI weibull mortality - browsed b */
  protected ModelVector mp_fHeightGLIWeibBrowsedB = new ModelVector(
      "Height-GLI Weibull - Browsed b", "mo_heightGLIWeibBrowsedB",
      "mo_hgwbbVal", 0, ModelVector.FLOAT);

  /** Height-GLI weibull mortality - browsed c */
  protected ModelVector mp_fHeightGLIWeibBrowsedC = new ModelVector(
      "Height-GLI Weibull - Browsed c", "mo_heightGLIWeibBrowsedC",
      "mo_hgwbcVal", 0, ModelVector.FLOAT);

  /** Height-GLI weibull mortality - browsed d */
  protected ModelVector mp_fHeightGLIWeibBrowsedD = new ModelVector(
      "Height-GLI Weibull - Browsed d", "mo_heightGLIWeibBrowsedD",
      "mo_hgwbdVal", 0, ModelVector.FLOAT);

  /** Height-GLI weibull mortality - browsed max mortality */
  protected ModelVector mp_fHeightGLIWeibBrowsedMaxMort = new ModelVector(
      "Height-GLI Weibull - Browsed Max Mortality (0 - 1)",
      "mo_heightGLIWeibBrowsedMaxMort", "mo_hgwbmmVal", 0, ModelVector.FLOAT);

  /** Exponential growth-resource mortality - "a"*/
  protected ModelVector mp_fExpResourceMortA = new ModelVector(
      "Exponential Growth-Resource - a", "mo_expResMortA",
      "mo_ermaVal", 0, ModelVector.FLOAT);

  /** Exponential growth-resource mortality - "b"*/
  protected ModelVector mp_fExpResourceMortB = new ModelVector(
      "Exponential Growth-Resource - b", "mo_expResMortB",
      "mo_ermbVal", 0, ModelVector.FLOAT);

  /** Exponential growth-resource mortality - "c"*/
  protected ModelVector mp_fExpResourceMortC = new ModelVector(
      "Exponential Growth-Resource - c", "mo_expResMortC",
      "mo_ermcVal", 0, ModelVector.FLOAT);

  /** Exponential growth-resource mortality - "d"*/
  protected ModelVector mp_fExpResourceMortD = new ModelVector(
      "Exponential Growth-Resource - d", "mo_expResMortD",
      "mo_ermdVal", 0, ModelVector.FLOAT);

  /**Browsed random juvenile mortality for each species*/
  protected ModelVector mp_fBrowsedJuvenileRandomMortality = new ModelVector(
      "Browsed Juvenile Background Mortality Rate", "mo_browsedRandomMortality",
      "mo_brmVal", 0, ModelVector.FLOAT); 

  /** Post harvest skidding mortality -Pre-harvest background mortality rate (beta)*/
  protected ModelVector mp_fPreHarvestBackgroundMort = new ModelVector(
      "Post Harvest Skid Mort - Pre-Harvest Background Mort Rate", 
      "mo_postHarvPreHarvestBackgroundMort", "mo_phphbmVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Windthrow harvest basic prob (rho w)*/
  protected ModelVector mp_fWindthrowHarvestBasicProb = new ModelVector(
      "Post Harvest Skid Mort - Windthrow Harvest Basic Prob", 
      "mo_postHarvWindthrowHarvestBasicProb", "mo_phwhbpVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Snag recruitment harvest basic prob (rho s)*/
  protected ModelVector mp_fSnagRecruitHarvestBasicProb = new ModelVector(
      "Post Harvest Skid Mort - Snag Recruitment Basic Prob", 
      "mo_postHarvSnagRecruitHarvestBasicProb", "mo_phsrhbpVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Windthrow size effect (delta w)*/
  protected ModelVector mp_fWindthrowSizeEffect = new ModelVector(
      "Post Harvest Skid Mort - Windthrow Size Effect", 
      "mo_postHarvWindthrowSizeEffect", "mo_phwseVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Windthrow harvest intensity effect (kappa w)*/
  protected ModelVector mp_fWindthrowHarvestIntensityEffect = new ModelVector(
      "Post Harvest Skid Mort - Windthrow Intensity Effect", 
      "mo_postHarvWindthrowHarvestIntensityEffect", "mo_phwhieVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Snag recruitment skidding effect (kappa s)*/
  protected ModelVector mp_fSnagRecruitHarvestIntensityEffect = new ModelVector(
      "Post Harvest Skid Mort - Snag Recruitment Skidding Effect", 
      "mo_postHarvSnagRecruitHarvestIntensityEffect", "mo_phsrhieVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Windthrow crowding effect (eta w)*/
  protected ModelVector mp_fWindthrowCrowdingEffect = new ModelVector(
      "Post Harvest Skid Mort - Windthrow Crowding Effect", 
      "mo_postHarvWindthrowCrowdingEffect", "mo_phwceVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Snag recruitment crowding effect (phi s)*/
  protected ModelVector mp_fSnagRecruitCrowdingEffect = new ModelVector(
      "Post Harvest Skid Mort - Snag Recruitment Crowding Effect", 
      "mo_postHarvSnagRecruitCrowdingEffect", "mo_phsrceVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Windthrow harvest rate param (tau w)*/
  protected ModelVector mp_fWindthrowHarvestRateParam = new ModelVector(
      "Post Harvest Skid Mort - Windthrow Harvest Rate Param", 
      "mo_postHarvWindthrowHarvestRateParam", "mo_phwhrpVal", 0, ModelVector.FLOAT);
  
  /** Post harvest skidding mortality -Snag recruitment harvest rate param (tau s)*/
  protected ModelVector mp_fSnagRecruitHarvestRateParam = new ModelVector(
      "Post Harvest Skid Mort - Snag Recruitment Rate Param", 
      "mo_postHarvSnagRecruitHarvestRateParam", "mo_phsrhrpVal", 0, ModelVector.FLOAT);
               
  /** Post harvest skidding mortality -windthrow Background Prob (omega)*/
  protected ModelVector mp_fWindthrowBackgroundProb = new ModelVector(
      "Post Harvest Skid Mort - Windthrow Background Prob", 
      "mo_postHarvWindthrowBackgroundProb", "mo_phwbpVal", 0, ModelVector.FLOAT);
               
  /** Post harvest skidding mortality -Snag recruitment background prob (zeta)*/
  protected ModelVector mp_fSnagRecruitBackgroundProb = new ModelVector(
      "Post Harvest Skid Mort - Snag Recruitment Background Prob", 
      "mo_postHarvSnagRecruitBackgroundProb", "mo_phsrbpVal", 0, ModelVector.FLOAT);
  
  /** Gompertz density self thinning mortality - G*/
  protected ModelVector mp_fGompertzG = new ModelVector(
      "Gompertz Density Self Thinning - G", 
      "mo_gompertzSelfThinningG", "mo_gstgVal", 0, ModelVector.FLOAT);
  
  /** Gompertz density self thinning mortality - H*/
  protected ModelVector mp_fGompertzH = new ModelVector(
      "Gompertz Density Self Thinning - H", 
      "mo_gompertzSelfThinningH", "mo_gsthVal", 0, ModelVector.FLOAT);
  
  /** Gompertz density self thinning mortality - I*/
  protected ModelVector mp_fGompertzI = new ModelVector(
      "Gompertz Density Self Thinning - I", 
      "mo_gompertzSelfThinningI", "mo_gstiVal", 0, ModelVector.FLOAT);
  
  /** Gompertz density self thinning mortality - min neighbor height*/
  protected ModelVector mp_fGompertzMinHeight = new ModelVector(
      "Gompertz Density Self Thinning - Min Neighbor Height (m)", 
      "mo_gompertzSelfThinningMinHeight", "mo_gstmhVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Adult Competition effect C */
  protected ModelVector mp_fWeibClimAdultCompC = new ModelVector(
      "Weibull Climate Survival - Adult Competition Effect \"C\"",
      "mo_weibClimAdultCompEffC", "mo_wcacecVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Competition effect C */
  protected ModelVector mp_fWeibClimJuvenileCompC = new ModelVector(
      "Weibull Climate Survival - Juvenile Competition Effect \"C\"",
      "mo_weibClimJuvCompEffC", "mo_wcjcecVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Competition effect D. */
  protected ModelVector mp_fWeibClimAdultCompD = new ModelVector(
      "Weibull Climate Survival - Adult Competition Effect \"D\"",
      "mo_weibClimAdultCompEffD", "mo_wcacedVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Competition effect D. */
  protected ModelVector mp_fWeibClimJuvenileCompD = new ModelVector(
      "Weibull Climate Survival - Juvenile Competition Effect \"D\"",
      "mo_weibClimJuvCompEffD", "mo_wcjcedVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Competition gamma*/
  protected ModelVector mp_fWeibClimAdultGamma = new ModelVector(
      "Weibull Climate Survival - Adult Competition Gamma",
      "mo_weibClimAdultCompEffGamma", "mo_wcacegVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Competition gamma*/
  protected ModelVector mp_fWeibClimJuvenileGamma = new ModelVector(
      "Weibull Climate Survival - Juvenile Competition Gamma",
      "mo_weibClimJuvCompEffGamma", "mo_wcjcegVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Maximum search radius, in meters, in which to 
   * look for crowding neighbors. */
  protected ModelVector mp_fWeibClimMaxCrowdingRadius = new ModelVector(
      "Weibull Climate Survival - Max Neighbor Search Radius (m)",
      "mo_weibClimMaxNeighRad", "mo_wcmnrVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - The minimum DBH, in cm, of neighbors to be 
   * included in the neighbor count.*/
  protected ModelVector mp_fWeibClimMinimumNeighborDBH = new ModelVector(
      "Weibull Climate Survival - Minimum Neighbor DBH (cm)",
      "mo_weibClimMinNeighDBH", "mo_wcmndVal", 0, ModelVector.FLOAT,true);

  /**Weibull climate survival - Adult Size effect X0.*/
  protected ModelVector mp_fWeibClimAdultSizeX0 = new ModelVector(
      "Weibull Climate Survival - Adult Size Effect X0",
      "mo_weibClimAdultSizeEffX0", "mo_wcasex0Val", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Size effect X0.*/
  protected ModelVector mp_fWeibClimJuvenileSizeX0 = new ModelVector(
      "Weibull Climate Survival - Juvenile Size Effect X0",
      "mo_weibClimJuvSizeEffX0", "mo_wcjsex0Val", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Size effect Xb.*/
  protected ModelVector mp_fWeibClimAdultSizeXb = new ModelVector(
      "Weibull Climate Survival - Adult Size Effect Xb",
      "mo_weibClimAdultSizeEffXb", "mo_wcasexbVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Size effect Xb.*/
  protected ModelVector mp_fWeibClimJuvenileSizeXb = new ModelVector(
      "Weibull Climate Survival - Juvenile Size Effect Xb",
      "mo_weibClimJuvSizeEffXb", "mo_wcjsexbVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Size effect minimum DBH.*/
  protected ModelVector mp_fWeibClimSizeMinDBH = new ModelVector(
      "Weibull Climate Survival - Size Effect Minimum DBH",
      "mo_weibClimSizeEffMinDBH", "mo_wcsemdVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Precipitation effect A.*/
  protected ModelVector mp_fWeibClimAdultPrecipA = new ModelVector(
      "Weibull Climate Survival - Adult Precip Effect \"A\"",
      "mo_weibClimAdultPrecipEffA", "mo_wcapeaVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Precipitation effect A.*/
  protected ModelVector mp_fWeibClimJuvenilePrecipA = new ModelVector(
      "Weibull Climate Survival - Juvenile Precip Effect \"A\"",
      "mo_weibClimJuvPrecipEffA", "mo_wcjpeaVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Precipitation effect B*/
  protected ModelVector mp_fWeibClimAdultPrecipB = new ModelVector(
      "Weibull Climate Survival - Adult Precip Effect \"B\"",
      "mo_weibClimAdultPrecipEffB", "mo_wcapebVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Precipitation effect B*/
  protected ModelVector mp_fWeibClimJuvenilePrecipB = new ModelVector(
      "Weibull Climate Survival - Juvenile Precip Effect \"B\"",
      "mo_weibClimJuvPrecipEffB", "mo_wcjpebVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Precipitation effect C.*/
  protected ModelVector mp_fWeibClimAdultPrecipC = new ModelVector(
      "Weibull Climate Survival - Adult Precip Effect \"C\"",
      "mo_weibClimAdultPrecipEffC", "mo_wcapecVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Precipitation effect C.*/
  protected ModelVector mp_fWeibClimJuvenilePrecipC = new ModelVector(
      "Weibull Climate Survival - Juvenile Precip Effect \"C\"",
      "mo_weibClimJuvPrecipEffC", "mo_wcjpecVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Temperature effect A.*/
  protected ModelVector mp_fWeibClimAdultTempA = new ModelVector(
      "Weibull Climate Survival - Adult Temp Effect \"A\"",
      "mo_weibClimAdultTempEffA", "mo_wcateaVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Temperature effect A.*/
  protected ModelVector mp_fWeibClimJuvenileTempA = new ModelVector(
      "Weibull Climate Survival - Juvenile Temp Effect \"A\"",
      "mo_weibClimJuvTempEffA", "mo_wcjteaVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Temperature effect B.*/
  protected ModelVector mp_fWeibClimAdultTempB = new ModelVector(
      "Weibull Climate Survival - Adult Temp Effect \"B\"",
      "mo_weibClimAdultTempEffB", "mo_wcatebVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Temperature effect B.*/
  protected ModelVector mp_fWeibClimJuvenileTempB = new ModelVector(
      "Weibull Climate Survival - Juvenile Temp Effect \"B\"",
      "mo_weibClimJuvTempEffB", "mo_wcjtebVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Temperature effect C. */
  protected ModelVector mp_fWeibClimAdultTempC = new ModelVector(
      "Weibull Climate Survival - Adult Temp Effect \"C\"",
      "mo_weibClimAdultTempEffC", "mo_wcatecVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Temperature effect C. */
  protected ModelVector mp_fWeibClimJuvenileTempC = new ModelVector(
      "Weibull Climate Survival - Juvenile Temp Effect \"C\"",
      "mo_weibClimJuvTempEffC", "mo_wcjtecVal", 0, ModelVector.FLOAT);

  /**Weibull climate survival - Adult Maximum potential survival probability */
  protected ModelVector mp_fWeibClimAdultMaxSurv = new ModelVector(
      "Weibull Climate Survival - Adult Max Survival Prob (0-1)",
      "mo_weibClimAdultMaxSurv", "mo_wcamsVal", 0, ModelVector.FLOAT);
  
  /**Weibull climate survival - Juvenile Maximum potential survival probability */
  protected ModelVector mp_fWeibClimJuvMaxSurv = new ModelVector(
      "Weibull Climate Survival - Juv Max Survival Prob (0-1)",
      "mo_weibClimJuvMaxSurv", "mo_wcjmsVal", 0, ModelVector.FLOAT);
  
  /**Temperature dependent neighborhood survival - A */
  protected ModelVector mp_fTempDepSurvA = new ModelVector(
      "Temp Dependent Neighborhood Surv - A",
      "mo_tempDepNeighA", "mo_tdnaVal", 0, ModelVector.FLOAT);
  
  /**Temperature dependent neighborhood survival - B */
  protected ModelVector mp_fTempDepSurvB = new ModelVector(
      "Temp Dependent Neighborhood Surv - B",
      "mo_tempDepNeighB", "mo_tdnbVal", 0, ModelVector.FLOAT);
  
  /**Temperature dependent neighborhood survival - M */
  protected ModelVector mp_fTempDepSurvM = new ModelVector(
      "Temp Dependent Neighborhood Surv - M",
      "mo_tempDepNeighM", "mo_tdnmVal", 0, ModelVector.FLOAT);
  
  /**Temperature dependent neighborhood survival - N */
  protected ModelVector mp_fTempDepSurvN = new ModelVector(
      "Temp Dependent Neighborhood Surv - N",
      "mo_tempDepNeighN", "mo_tdnnVal", 0, ModelVector.FLOAT);
  
  /**Suppression duration mortality - Maximum mortality rate*/
  protected ModelVector mp_fSuppDurrMortMax = new ModelVector(
      "Suppression Duration Mortality - Max Mortality Rate (0-1)", 
      "mo_suppDurMaxMort", "mo_sdmmVal", 0, ModelVector.FLOAT);
  
  /**Suppression duration mortality - X0*/
  protected ModelVector mp_fSuppDurrMortX0 = new ModelVector(
      "Suppression Duration Mortality - X0", 
      "mo_suppDurMortX0", "mo_sdmx0Val", 0, ModelVector.FLOAT);
  
  /**Suppression duration mortality - Xb*/
  protected ModelVector mp_fSuppDurrMortXb = new ModelVector(
      "Suppression Duration Mortality - Xb", 
      "mo_suppDurMortXb", "mo_sdmxbVal", 0, ModelVector.FLOAT);
  
  /**Insect Mortality - Intercept - initial mortality rate*/
  protected ModelVector mp_fInsectMortIntercept = new ModelVector(
      "Insect Mortality - Intercept", "mo_insectMortIntercept", "mo_imiVal", 
      0, ModelVector.FLOAT);

  /**Insect Mortality - Maximum mortality rate*/
  protected ModelVector mp_fInsectMortMax = new ModelVector(
      "Insect Mortality - Max Mortality Rate (0-1)", "mo_insectMortMaxRate", 
      "mo_immrVal", 0, ModelVector.FLOAT);

  /**Insect Mortality - X0*/
  protected ModelVector mp_fInsectMortX0 = new ModelVector(
      "Insect Mortality - X0", "mo_insectMortX0", "mo_imx0Val", 
      0, ModelVector.FLOAT);

  /**Insect Mortality - Xb*/
  protected ModelVector mp_fInsectMortXb = new ModelVector(
      "Insect Mortality - Xb", "mo_insectMortXb", "mo_imxbVal", 
      0, ModelVector.FLOAT);
  
  /** Temperature dependent neighborhood survival - neighbor search radius*/
  protected ModelFloat m_fTempDepSurvSearchRadius = new ModelFloat(
      "Temp Dependent Neighborhood Surv - Neigh Search Radius (m)", 
      "mo_tempDepNeighRadius"); 
  
  /** Gompertz density self thinning mortality - neighbor search radius*/
  protected ModelFloat m_fGompertzSearchRadius = new ModelFloat(
      "Gompertz Density Self Thinning - Neighbor Search Radius (m)", 
      "mo_gompertzSelfThinningRadius"); 
               
  /** Post harvest skidding mortality -Crowding effect radius*/
  protected ModelFloat m_fCrowdingEffectRadius = new ModelFloat(
      "Post Harvest Skid Mort - Crowding Effect Radius", 
      "mo_postHarvCrowdingEffectRadius"); 

  /**NCI - NCI DBH divisor*/
  protected ModelFloat m_fNCIDbhDivisor = new ModelFloat(1,
      "NCI DBH Divisor (q)", "mo_nciDbhDivisor");

  /** Aggregated mortality - mortality episode return interval in years*/
  protected ModelFloat m_fAggMortEpisodeReturnInterval =
      new ModelFloat(1, "Aggregated Mortality Return Interval (years)",
                     "mo_aggReturnInterval");

  /** Aggregated mortality - mortality rate per year of a
   * mortality episode, 0-1 */
  protected ModelFloat m_fAggMortPropToKill =
      new ModelFloat(0, "Aggregated Mortality Annual Kill Amount (0-1)",
                     "mo_aggPropTreesToKill");

  /** Aggregated mortality - number of trees to aggregate */
  protected ModelFloat m_fAggMortNumTreesToAggregate =
      new ModelFloat(1, "Aggregated Mortality Number of Trees To Aggregate",
                     "mo_aggNumTreesToClump");

  /** Aggregated mortality - clumping parameter for negative binomial
   * distribution, if required */
  protected ModelFloat m_fAggMortClumpingParameter =
      new ModelFloat(1, "Aggregated Mortality Clumping Parameter",
                     "mo_aggClumpingParameter");

  /** Aggregated mortality - whether clump size is deterministic (true) or
    * from the negative binomial probability distribution (false) */
  protected ModelEnum m_iAggMortClumpDeterministic =
      new ModelEnum(new int[] {1, 0},
                    new String[] {"Deterministic", "Neg. binomial"},
                    "Aggregated Mortality Clump Size",
                    "mo_aggClumpSizeDeterministic");

  /**NCI - Whether or not to include snags in NCI calculations - LEM 05/12/05*/
  protected ModelEnum m_iIncludeSnagsInNCI =
      new ModelEnum(new int[] {0, 1},
                    new String[] {"false", "true"},
                    "Include Snags in NCI Calculations",
                    "mo_nciIncludeSnagsInNCI");
  
  /**Harvest mortality reason code. Matches C++ code.*/  
  public final static int harvest = 1; 
  /**Natural mortality reason code. Matches C++ code.*/
  public final static int natural = 2;
  /**Disease mortality reason code. Matches C++ code.*/
  public final static int disease = 3;
  /**Fire mortality reason code. Matches C++ code.*/
  public final static int fire = 4;
  /**Insects mortality reason code. Matches C++ code.*/
  public final static int insects = 5;
  /**Storm mortality reason code. Matches C++ code.*/
  public final static int storm = 6;
 
    


  /**
   * Constructor.
   * @param oManager GUIManager object.
   * @throws ModelException passed through from called functions.  Should never
   * be thrown.
   */
  public MortalityBehaviors(GUIManager oManager) throws ModelException {
    super(oManager, "Mortality", "mortality");

    m_sHelpID = "data.mortality_behaviors";

    //Set up child behaviors
    mp_oChildBehaviors = new Behavior[25];

    int iIndex = 0;

    //*****************
     //Adult self-thinning
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("adultselfthin", "Adult Self Thinning",
                                         "adultselfthin",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultSelfThinningSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultSelfThinningIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultSelfThinningMaxDbh);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SAPLING, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Adult stochastic mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("adultstochmort",
                                         "Adult Stochastic Mortality",
                                         "adultstochasticmort",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fAdultRandomMortality);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Aggregated mortality - 07-28-06
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("aggregated mortality",
                                         "Aggregated Mortality",
                                         "Aggregated Mortality",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fAggMortEpisodeReturnInterval);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fAggMortPropToKill);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fAggMortNumTreesToAggregate);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fAggMortClumpingParameter);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iAggMortClumpDeterministic);

    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;


    //*****************
     //BC mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("bcmort", "BC Mortality",
                                         "bcmortality",
                                          1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMortAtZeroGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLightDependentMortality);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Browsed stochastic mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("browsed stochastic mortality",
                                              "Browsed Stochastic Mortality",
                                              "Browsed Stochastic Mortality",
                                          1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fJuvenileRandomMortality);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fBrowsedJuvenileRandomMortality);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Competition mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("competition mortality",
                                          "Competition Mortality",
                                          "competitionmortality",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompMortShape);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fCompMortMax);
    //Can't apply to seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Density self-thinning - LEM 03-17-2005
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("density self-thinning mortality",
                                          "Density Self-Thinning Mortality",
                                          "densityselfthinning",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDensSelfThinAsymptote);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDensSelfThinDensityEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDensSelfThinDiamEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDensSelfThinMinDensity);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDensSelfThinNeighRadius);
    //Can't apply to adults
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.ADULT, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Exponential growth and resource based mortality - LEM 07-06-06
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("exp resource mortality",
                                         "Exponential Growth and Resource-Based Mortality",
                                         "Exponential growth resource mortality",
                                          1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fExpResourceMortA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fExpResourceMortB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fExpResourceMortC);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fExpResourceMortD);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;


    //*****************
    //GMF mortality
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("gmfmort", "GMF Mortality",
                                         "gmfmortality",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fMortAtZeroGrowth);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLightDependentMortality);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;
    
    //*****************
    //Gompertz density self thinning mortality
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("Gompertz Density Self Thinning", 
                                              "Gompertz Density Self Thinning",
                                        "Gompertz Density Self Thinning",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fGompertzG);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fGompertzH);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fGompertzI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fGompertzMinHeight);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fGompertzSearchRadius);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
       "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Growth and resource based mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("resource mortality",
                                         "Growth and Resource-Based Mortality",
                                         "Growth resource mortality",
                                          1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fResMortFunctionMode);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fResMortGrowthIncSurv);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fResMortLoGrowthShape);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fResMortScalingFactor);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
    //Height-GLI Weibull mortality
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("height-gli weibull mortality",
                                          "Height-GLI Weibull Mortality with Browse",
                                          "Height GLI Weibull Mortality",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibMaxMort);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibC);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibD);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibBrowsedMaxMort);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibBrowsedA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibBrowsedB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibBrowsedC);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fHeightGLIWeibBrowsedD);

    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;
    
    //*****************
    //Insect infestation mortality
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("insect mortality",
                                          "Insect Infestation Mortality",
                                          "Insect Infestation Mortality",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fInsectMortIntercept);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fInsectMortMax);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fInsectMortX0);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fInsectMortXb);
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Juvenile self-thinning
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("juvselfthin",
                                         "Juvenile Self Thinning",
                                         "juvselfthin",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fJuvenileSelfThinningSlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fJuvenileSelfThinningIntercept);
    //Disallow for adults
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.ADULT, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Juvenile stochastic mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("juvstochmort",
                                         "Juvenile Stochastic Mortality",
                                         "juvstochasticmort",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fJuvenileRandomMortality);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Logistic bi-level - LEM 05-06-2005
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("logistic bi-level mortality",
                                          "Logistic Bi-Level Mortality",
                                          "Logistic Bi-Level Mortality",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevHiLiteThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevLoLiteA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevLoLiteB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevHiLiteA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fLogBiLevHiLiteB);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //NCI mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("NCI Mortality",
                                         "NCI Mortality",
                                         "NCI Mortality",
                                          2.0);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxCrowdingRadius);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighDBHEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighDistanceEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeSensToNCI);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMaxProbSurvival);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectMode);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISizeEffectVariance);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISlope);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCISteepness);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIStormEffectMed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIStormEffectFull);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighStormEffMed);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCINeighStormEffFull);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIMinNeighborDBH);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIShadingEffectCoefficient);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fNCIShadingEffectExponent);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fNCIDbhDivisor);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iIncludeSnagsInNCI);

    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;
    
    //*****************
    //Post harvest skidding mortality 
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("post harvest skidding mortality",
        "Post harvest skidding mortality",
        "postharvestskiddingmortality",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fPreHarvestBackgroundMort);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindthrowHarvestBasicProb);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnagRecruitHarvestBasicProb);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindthrowSizeEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindthrowHarvestIntensityEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnagRecruitHarvestIntensityEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindthrowCrowdingEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnagRecruitCrowdingEffect);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindthrowHarvestRateParam);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnagRecruitHarvestRateParam);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWindthrowBackgroundProb);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnagRecruitBackgroundProb);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_fCrowdingEffectRadius); 
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
     //Senescense
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("senescence", "Senescence",
                                         "senescence",  1);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRandomMortalityAlpha);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fRandomMortalityBeta);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fDbhAtOnsetOfSenescence);
    mp_oChildBehaviors[iIndex].AddRequiredData(m_iDbhAtAsymptoticMaximumMortality);
    //Disallow for seedlings
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    //*****************
    //Stochastic bi-level - storm light 
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("stochastic bi-level mortality",
                                          "Stochastic Bi-Level Mort - Storm Light",
                                          "Stochastic Bi-Level Mortality",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStochBiLevHiLiteThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStochBiLevLoLiteMortProb);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStochBiLevHiLiteMortProb);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;
    
    //*****************
    //Stochastic bi-level - GLI
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("stochastic bi-level mortality gli",
                                          "Stochastic Bi-Level Mort - GLI",
                                          "Stochastic Bi-Level Mortality - GLI",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStochBiLevHiLiteThreshold);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStochBiLevLoLiteMortProb);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fStochBiLevHiLiteMortProb);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;
    
    //*****************
    //Suppression duration
    //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("suppression duration mortality",
                                          "Suppression Duration Mortality",
                                          "Suppression Duration Mortality",
                                           1);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSuppDurrMortMax);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSuppDurrMortX0);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSuppDurrMortXb);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;
    
    //*****************
    //Temp Dependent Neighborhood Survival
    //*****************
     mp_oChildBehaviors[iIndex] = new Behavior("temp dependent neigh surv",
                                           "Temperature Dependent Neighborhood Survival",
                                           "Temp Dependent Neighborhood Survival",
                                            1);
     mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTempDepSurvA);
     mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTempDepSurvB);
     mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTempDepSurvM);
     mp_oChildBehaviors[iIndex].AddRequiredData(mp_fTempDepSurvN);
     mp_oChildBehaviors[iIndex].AddRequiredData(m_fTempDepSurvSearchRadius);
     //Data members
     mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
         "dead", DataMember.INTEGER));
     iIndex++;
    
   //*****************
   //Weibull climate survival 
   //*****************
   mp_oChildBehaviors[iIndex] = new Behavior("weibull climate survival",
                                         "Weibull Climate Survival",
                                         "WeibullClimateSurvival",
                                          1);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultMaxSurv);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvMaxSurv);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultCompC);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultCompD);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultGamma);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileCompC);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileCompD);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileGamma);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimMaxCrowdingRadius);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimMinimumNeighborDBH);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultSizeX0);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultSizeXb);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileSizeX0);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileSizeXb);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimSizeMinDBH);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultPrecipA);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultPrecipB);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultPrecipC);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenilePrecipA);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenilePrecipB);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenilePrecipC);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultTempA);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultTempB);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimAdultTempC);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileTempA);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileTempB);
   mp_oChildBehaviors[iIndex].AddRequiredData(mp_fWeibClimJuvenileTempC);
   mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
   //Data members
   mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
       "dead", DataMember.INTEGER));
   iIndex++;

    //*****************
     //Weibull snag mortality
     //*****************
    mp_oChildBehaviors[iIndex] = new Behavior("weibull snag mortality",
                                         "Weibull Snag Mortality",
                                         "weibull snag mortality",  1.1);
    //Set to only apply for snags
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SEEDLING, false);
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SAPLING, false);
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.ADULT, false);
    mp_oChildBehaviors[iIndex].SetCanApplyTo(TreePopulation.SNAG, true);
    //Data
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnag1WeibullA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnag2WeibullA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnag3WeibullA);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnag1WeibullB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnag2WeibullB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnag3WeibullB);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnagSizeClass1Dbh);
    mp_oChildBehaviors[iIndex].AddRequiredData(mp_fSnagSizeClass2Dbh);
    //Data members
    mp_oChildBehaviors[iIndex].mp_oNewTreeDataMembers.add(new DataMember("Dead flag",
        "dead", DataMember.INTEGER));
    iIndex++;

    mp_oAllData.add(mp_fJuvenileRandomMortality);
    mp_oAllData.add(mp_fAdultRandomMortality);
    mp_oAllData.add(mp_fMortAtZeroGrowth);
    mp_oAllData.add(mp_fLightDependentMortality);
    mp_oAllData.add(mp_fJuvenileSelfThinningSlope);
    mp_oAllData.add(mp_fJuvenileSelfThinningIntercept);
    mp_oAllData.add(mp_fAdultSelfThinningSlope);
    mp_oAllData.add(mp_fAdultSelfThinningIntercept);
    mp_oAllData.add(mp_fAdultSelfThinningMaxDbh);
    mp_oAllData.add(mp_fRandomMortalityAlpha);
    mp_oAllData.add(mp_fRandomMortalityBeta);
    mp_oAllData.add(mp_fDbhAtOnsetOfSenescence);
    mp_oAllData.add(m_iDbhAtAsymptoticMaximumMortality);
    mp_oAllData.add(mp_fSnag1WeibullA);
    mp_oAllData.add(mp_fSnag2WeibullA);
    mp_oAllData.add(mp_fSnag3WeibullA);
    mp_oAllData.add(mp_fSnag1WeibullB);
    mp_oAllData.add(mp_fSnag2WeibullB);
    mp_oAllData.add(mp_fSnag3WeibullB);
    mp_oAllData.add(mp_fSnagSizeClass1Dbh);
    mp_oAllData.add(mp_fSnagSizeClass2Dbh);
    mp_oAllData.add(mp_fNCIMaxCrowdingRadius);
    mp_oAllData.add(mp_fNCINeighDBHEffect);
    mp_oAllData.add(mp_fNCINeighDistanceEffect);
    mp_oAllData.add(mp_fNCISizeSensToNCI);
    mp_oAllData.add(mp_fNCIMaxProbSurvival);
    mp_oAllData.add(mp_fNCISizeEffectMode);
    mp_oAllData.add(mp_fNCISizeEffectVariance);
    mp_oAllData.add(mp_fNCISlope);
    mp_oAllData.add(mp_fNCISteepness);
    mp_oAllData.add(mp_fNCIStormEffectMed);
    mp_oAllData.add(mp_fNCIStormEffectFull);
    mp_oAllData.add(mp_fNCINeighStormEffMed);
    mp_oAllData.add(mp_fNCINeighStormEffFull);
    mp_oAllData.add(mp_fNCIMinNeighborDBH);
    mp_oAllData.add(mp_fNCIShadingEffectCoefficient);
    mp_oAllData.add(mp_fNCIShadingEffectExponent);
    mp_oAllData.add(m_fNCIDbhDivisor);
    mp_oAllData.add(m_iIncludeSnagsInNCI);
    mp_oAllData.add(mp_fResMortFunctionMode);
    mp_oAllData.add(mp_fResMortGrowthIncSurv);
    mp_oAllData.add(mp_fResMortLoGrowthShape);
    mp_oAllData.add(mp_fResMortScalingFactor);
    mp_oAllData.add(mp_fCompMortShape);
    mp_oAllData.add(mp_fCompMortMax);
    mp_oAllData.add(mp_fDensSelfThinAsymptote);
    mp_oAllData.add(mp_fDensSelfThinDensityEffect);
    mp_oAllData.add(mp_fDensSelfThinDiamEffect);
    mp_oAllData.add(mp_fDensSelfThinMinDensity);
    mp_oAllData.add(mp_fDensSelfThinNeighRadius);
    mp_oAllData.add(mp_fLogBiLevHiLiteThreshold);
    mp_oAllData.add(mp_fLogBiLevLoLiteA);
    mp_oAllData.add(mp_fLogBiLevLoLiteB);
    mp_oAllData.add(mp_fLogBiLevHiLiteA);
    mp_oAllData.add(mp_fLogBiLevHiLiteB);
    mp_oAllData.add(mp_fStochBiLevHiLiteThreshold);
    mp_oAllData.add(mp_fStochBiLevLoLiteMortProb);
    mp_oAllData.add(mp_fStochBiLevHiLiteMortProb);
    mp_oAllData.add(mp_fHeightGLIWeibMaxMort);
    mp_oAllData.add(mp_fHeightGLIWeibA);
    mp_oAllData.add(mp_fHeightGLIWeibB);
    mp_oAllData.add(mp_fHeightGLIWeibC);
    mp_oAllData.add(mp_fHeightGLIWeibD);
    mp_oAllData.add(mp_fExpResourceMortA);
    mp_oAllData.add(mp_fExpResourceMortB);
    mp_oAllData.add(mp_fExpResourceMortC);
    mp_oAllData.add(mp_fExpResourceMortD);
    mp_oAllData.add(m_fAggMortEpisodeReturnInterval);
    mp_oAllData.add(m_fAggMortPropToKill);
    mp_oAllData.add(m_fAggMortNumTreesToAggregate);
    mp_oAllData.add(m_fAggMortClumpingParameter);
    mp_oAllData.add(m_iAggMortClumpDeterministic);
    mp_oAllData.add(mp_fBrowsedJuvenileRandomMortality);
    mp_oAllData.add(mp_fHeightGLIWeibBrowsedMaxMort);
    mp_oAllData.add(mp_fHeightGLIWeibBrowsedA);
    mp_oAllData.add(mp_fHeightGLIWeibBrowsedB);
    mp_oAllData.add(mp_fHeightGLIWeibBrowsedC);
    mp_oAllData.add(mp_fHeightGLIWeibBrowsedD);
    mp_oAllData.add(mp_fPreHarvestBackgroundMort);
    mp_oAllData.add(mp_fWindthrowHarvestBasicProb);
    mp_oAllData.add(mp_fSnagRecruitHarvestBasicProb);
    mp_oAllData.add(mp_fWindthrowSizeEffect);
    mp_oAllData.add(mp_fWindthrowHarvestIntensityEffect);
    mp_oAllData.add(mp_fSnagRecruitHarvestIntensityEffect);
    mp_oAllData.add(mp_fWindthrowCrowdingEffect);
    mp_oAllData.add(mp_fSnagRecruitCrowdingEffect);
    mp_oAllData.add(mp_fWindthrowHarvestRateParam);
    mp_oAllData.add(mp_fSnagRecruitHarvestRateParam);
    mp_oAllData.add(mp_fWindthrowBackgroundProb);
    mp_oAllData.add(mp_fSnagRecruitBackgroundProb);
    mp_oAllData.add(m_fCrowdingEffectRadius); 
    mp_oAllData.add(mp_fGompertzG);
    mp_oAllData.add(mp_fGompertzH);
    mp_oAllData.add(mp_fGompertzI);
    mp_oAllData.add(mp_fGompertzMinHeight);
    mp_oAllData.add(m_fGompertzSearchRadius);
    mp_oAllData.add(mp_fWeibClimAdultMaxSurv);
    mp_oAllData.add(mp_fWeibClimJuvMaxSurv);
    mp_oAllData.add(mp_fWeibClimAdultCompC);
    mp_oAllData.add(mp_fWeibClimAdultCompD);
    mp_oAllData.add(mp_fWeibClimAdultGamma);
    mp_oAllData.add(mp_fWeibClimJuvenileCompC);
    mp_oAllData.add(mp_fWeibClimJuvenileCompD);
    mp_oAllData.add(mp_fWeibClimJuvenileGamma);
    mp_oAllData.add(mp_fWeibClimMaxCrowdingRadius);
    mp_oAllData.add(mp_fWeibClimMinimumNeighborDBH);
    mp_oAllData.add(mp_fWeibClimAdultSizeX0);
    mp_oAllData.add(mp_fWeibClimAdultSizeXb);
    mp_oAllData.add(mp_fWeibClimJuvenileSizeX0);
    mp_oAllData.add(mp_fWeibClimJuvenileSizeXb);
    mp_oAllData.add(mp_fWeibClimSizeMinDBH);
    mp_oAllData.add(mp_fWeibClimAdultPrecipA);
    mp_oAllData.add(mp_fWeibClimAdultPrecipB);
    mp_oAllData.add(mp_fWeibClimAdultPrecipC);
    mp_oAllData.add(mp_fWeibClimJuvenilePrecipA);
    mp_oAllData.add(mp_fWeibClimJuvenilePrecipB);
    mp_oAllData.add(mp_fWeibClimJuvenilePrecipC);
    mp_oAllData.add(mp_fWeibClimAdultTempA);
    mp_oAllData.add(mp_fWeibClimAdultTempB);
    mp_oAllData.add(mp_fWeibClimAdultTempC);
    mp_oAllData.add(mp_fWeibClimJuvenileTempA);
    mp_oAllData.add(mp_fWeibClimJuvenileTempB);
    mp_oAllData.add(mp_fWeibClimJuvenileTempC);
    mp_oAllData.add(mp_fTempDepSurvA);
    mp_oAllData.add(mp_fTempDepSurvB);
    mp_oAllData.add(mp_fTempDepSurvM);
    mp_oAllData.add(mp_fTempDepSurvN);
    mp_oAllData.add(m_fTempDepSurvSearchRadius);
    mp_oAllData.add(mp_fInsectMortIntercept);
    mp_oAllData.add(mp_fInsectMortMax);
    mp_oAllData.add(mp_fInsectMortX0);
    mp_oAllData.add(mp_fInsectMortXb);
    mp_oAllData.add(mp_fSuppDurrMortMax);
    mp_oAllData.add(mp_fSuppDurrMortX0);
    mp_oAllData.add(mp_fSuppDurrMortXb);
    

    //Grids
    Grid oNewGrid;
    //mp_oGrids = new Grid[1];
    //Resource grid
    String sGridName = "Resource";

    //Create the data member
    DataMember[] p_oDataMembers = new DataMember[1];

    p_oDataMembers[0] = new DataMember("Resource", "Resource", DataMember.FLOAT);
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);

    //Assign to resource mortality behaviors
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("resource mortality").AddGrid(oNewGrid);
    GetBehaviorByKey("exp resource mortality").AddGrid(oNewGrid);
    
    //Years since last harvest grid 
    sGridName = "Years Since Last Harvest"; 
    p_oDataMembers = new DataMember[2];
    p_oDataMembers[0] = new DataMember("Time", "Time", DataMember.INTEGER);
    p_oDataMembers[1] = new DataMember("LastUpdated", "LastUpdated", DataMember.INTEGER);
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 8, 8);
    //Assign to post harvest skidding mortality 
    GetBehaviorByKey("post harvest skidding mortality").AddGrid(oNewGrid);
    

  }

  /**
   * Adds NCI lambda values.
   * @param oPop Tree population
   * @throws ModelException Not thrown.
   */
  public void DoSetup(TreePopulation oPop) throws ModelException {
    Behavior oBeh = GetBehaviorByKey("NCI Mortality");
    int iNumSpecies = oPop.GetNumberOfSpecies(), i, j;
    boolean bAlreadyLambdas = false;

    //See if we already had lambdas
    for (i = 0; i < mp_oAllData.size(); i++) {
      ModelData oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor().toLowerCase();
      if (sKey.indexOf("lambda neighbor") > -1) {
        bAlreadyLambdas = true;
        break;
      }
    }

    if (bAlreadyLambdas == false) {

      for (i = 0; i < iNumSpecies; i++) {
        String sSpName = oPop.GetSpeciesNameFromCode(i);
        ModelVector oVector = new ModelVector(
            sSpName.replace('_', ' ') + " NCI Lambda Neighbors",
            "mo_prnci" + sSpName + "NeighborLambda", "mo_plVal",
            0, ModelVector.FLOAT);

        mp_oAllData.add(oVector);
        oBeh.AddRequiredData(oVector);
      }
    }
    else {
      //We already had lambdas.  What we want to do is preserve any for species
      //that didn't change, because they may have values.

      //Create an array of the lambda arrays, placed in the new species
      //order
      ModelData[] p_oNCI = new ModelData[iNumSpecies];
      int iCode;

      for (i = 0; i < iNumSpecies; i++) {
        p_oNCI[i] = null;
      }

      //Get the existing lambdas and put them where they go in our arrays -
      //any for species that were removed will be left behind
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor().toLowerCase();
        if (sKey.indexOf("lambda neighbor") > -1) {
          //get the species
          String sSpecies;
          sKey = oData.GetDescriptor();
          sSpecies = sKey.substring(0, sKey.indexOf(" NCI")).replace(' ', '_');
          iCode = oPop.GetSpeciesCodeFromName(sSpecies);
          if (iCode > -1) {
            p_oNCI[iCode] = oData;
          }
        }
      }

      //Now remove all lambdas from the required data and from all behaviors
      for (i = 0; i < mp_oAllData.size(); i++) {
        ModelData oData = (ModelData) mp_oAllData.elementAt(i);
        String sKey = oData.GetDescriptor().toLowerCase();
        if (sKey.indexOf("lambda neighbor") > -1) {
          mp_oAllData.remove(i);
          i--;
        }
      }
      for (i = 0; i < mp_oChildBehaviors.length; i++) {
        for (j = 0; j < mp_oChildBehaviors[i].mp_oRequiredData.size(); j++) {
          ModelData oData = (ModelData) mp_oChildBehaviors[i].
              mp_oRequiredData.elementAt(j);
          String sKey = oData.GetDescriptor().toLowerCase();
          if (sKey.indexOf("lambda neighbor") > -1) {
            mp_oChildBehaviors[i].mp_oRequiredData.remove(j);
            j--;
          }
        }
      }

      //Now add back the lambdas, creating new ones for any new species
      for (i = 0; i < p_oNCI.length; i++) {
        if (p_oNCI[i] == null) {
          String sSpName = oPop.GetSpeciesNameFromCode(i);
          p_oNCI[i] = new ModelVector(
              sSpName.replace('_', ' ') + " NCI Lambda Neighbors",
              "mo_prnci" + sSpName + "NeighborLambda", "mo_plVal",
              0, ModelVector.FLOAT);
        }
        mp_oAllData.add(p_oNCI[i]);
        oBeh.AddRequiredData(p_oNCI[i]);
      }
    }

    //Now set new NCI arrays to 0 to allow easy backwards compatibility -
    //remove this section later.  Setting them to 0 allows a simple save to
    //write a new, compatible file.
    //LEM 03-16-05
    if (mp_fNCIMinNeighborDBH.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIMinNeighborDBH.GetValue().add(new Float(0.0));
      }
    }
    if (mp_fNCIShadingEffectCoefficient.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIShadingEffectCoefficient.GetValue().add(new Float(0.0));
      }
    }
    if (mp_fNCIShadingEffectExponent.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIShadingEffectExponent.GetValue().add(new Float(0.0));
      }
    }

    //If the damage parameters have not already been set up, set all their
    //values to a default of 1.  Otherwise leave them alone.
    if (mp_fNCINeighStormEffMed.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCINeighStormEffMed.GetValue().add(new Float(1.0));
      }
    }

    if (mp_fNCINeighStormEffFull.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCINeighStormEffFull.GetValue().add(new Float(1.0));
      }
    }

    if (mp_fNCIStormEffectMed.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIStormEffectMed.GetValue().add(new Float(1.0));
      }
    }

    if (mp_fNCIStormEffectFull.GetValue().size() == 0) {
      for (i = 0; i < iNumSpecies; i++) {
        mp_fNCIStormEffectFull.GetValue().add(new Float(1.0));
      }
    }
    
    // Set up the grid for Temperature dependent neighborhood survival 
    DataMember[] p_oDataMembers = new DataMember[iNumSpecies+1];
    Grid oNewGrid;
    String sGridName = "Temperature Dependent Neighborhood Survival";

    // The accessible data members are a value for each species
    for (i = 0; i < iNumSpecies; i++) {
      p_oDataMembers[i] = new DataMember("Survival for "
          + oPop.GetSpeciesNameFromCode(i).replace('_', ' '), "survival_" + i,
          DataMember.FLOAT);
    }
    p_oDataMembers[iNumSpecies] = new DataMember("Neighbor basal area", 
        "BAT", DataMember.FLOAT); 

    // Create our grid
    oNewGrid = new Grid(sGridName, p_oDataMembers, null, 2, 2);

    // Assign the grid to the Temperature dependent neighborhood survival 
    // behavior
    oNewGrid = m_oManager.AddGrid(oNewGrid);
    GetBehaviorByKey("temp dependent neigh surv").AddGrid(oNewGrid);
  }

  /**
   * Overridden to provide backwards compatibility for Weibull snag mortality
   * @param sXMLTag XML tag of data object whose value is to be set.
   * @param sXMLParentTag The immediate parent tag that sXMLTag is within.
   * @param oAttributes Attributes of the object.  Ignored, but may be needed
   * by overriding objects.
   * @param oData Data value, either String or type appropriate to the data
   * type
   * @return true if the value was set successfully; false if the value could
   * not be found.  (This would not be an error, because I need a way to
   * cycle through the objects until one of the objects comes up with a
   * match.)
   * @throws ModelException if the value could not be assigned to the data
   * object.
   */
  public boolean SetSingleValueByXMLTag(String sXMLTag, String sXMLParentTag,
                                        org.xml.sax.Attributes oAttributes,
                                        Object oData) throws
      ModelException {
    boolean bFlag = super.SetSingleValueByXMLTag(sXMLTag, sXMLParentTag,
                                                 oAttributes, oData);
    if (bFlag == true) {
      //A value was assigned - see if it was one of the weibull snag size
      //classes
      if (sXMLTag.equals(m_fSnagSizeClass1DbhDep.GetXMLTag())) {
        //Cross-set this value into the species-specific version
        Float fVal = null;
        if (oData instanceof String) {
          fVal = new Float( (String) oData);
        }
        else {
          fVal = (Float) oData;
        }
        int iNumSpecies = m_oManager.GetTreePopulation().GetNumberOfSpecies();
        Float[] p_fVals = new Float[iNumSpecies];
        for (int i = 0; i < p_fVals.length; i++) {
          p_fVals[i] = fVal;
        }
        SetVectorValues(mp_fSnagSizeClass1Dbh, p_fVals);
      }
      else if (sXMLTag.equals(m_fSnagSizeClass2DbhDep.GetXMLTag())) {
        //Cross-set this value into the species-specific version
        Float fVal = null;
        if (oData instanceof String) {
          fVal = new Float( (String) oData);
        }
        else {
          fVal = (Float) oData;
        }
        int iNumSpecies = m_oManager.GetTreePopulation().GetNumberOfSpecies();
        Float[] p_fVals = new Float[iNumSpecies];
        for (int i = 0; i < p_fVals.length; i++) {
          p_fVals[i] = fVal;
        }
        SetVectorValues(mp_fSnagSizeClass2Dbh, p_fVals);
      }
    }
    return bFlag;
  }

  /**
   * Validates the data prior to writing it to a parameter file or some such.
   * @param oPop TreePopulation
   * @throws ModelException if:
   * <ul>
   * <li>Any of the following are not expressed as a proportion:
   * <ul>
   * <li>mp_fAdultRandomMortality
   * <li>mp_fJuvenileRandomMortality
   * <li>mp_fMortAtZeroGrowth
   * </ul></li>
   * <li>If adult self-thinning is used and any mp_fAdultSelfThinningMaxDbh
   * value is less than zero.</li>
   * <li>Any  value in mp_fDbhAtOnsetOfSenescence is less than zero.</li>
   * <li>Any vector is not sized correctly to equal total number of
   * species.</li>
   * <li>GMF mortality is enabled and the timestep length is not five
   * years.</li>
   * <li>Any value in light dependent mortality is zero.</li>
   * <li>Competition mortality is enabled and the timestep length is not one
   * year.</li>
   * <li>Competition mortality is enabled and NCI growth is not.</li>
   * <li>NCI Mortality is enabled and any of the following is true:
   * <ul>
   * <li>NCI Max. radius of neighbor effects is <= 0 for any species</li>
   * <li>NCI Max growth is <= 0 for any species</li>
   * <li>NCI size effect mode for any species is <= 0</li>
   * <li>NCI Size effect variance = 0 for any species</li>
   * <li>NCI Eta for any species for any damage category is not between 0 and
   * 1</li>
   * <li>NCI Storm Effect parameters for any species for any damage category
   * are not between 0 and 1</li>
   * <li>NCI DBH divisor is <= 0</li>
   * <li>Minimun neighbor DBH value is < 0</li>
   * </ul></li>
   * <li>Density self-thinning is enabled, and any of the following are true:
   * <ul>
   * <li>The timestep length is greater than 1.</li>
   * <li>The neighborhood radius is not greater than 0.</li>
   * <li>The minimum density for mortality is less than 0.</li>
   * <li>The self-thinning asymptote is not greater than 0.</li>
   * <li>The density effect is not greater than 0.</li>
   * </ul></li>
   * <li>Logistic bi-level is enabled, and the threshold for high-light
   * mortality is not between 0 and 100.</li>
   * <li>Stochastic bi-level is enabled, and the threshold for high-light
   * mortality is not between 0 and 100.</li>
   * <li>Stochastic bi-level is enabled, and values for high- and low-light
   * mortality probability are not between 0 and 1.</li>
   * <li>Stochastic bi-level is enabled, and Storm Light is not.</li>
   * <li>Weibull snag mortality is enabled, and either a is 0 or a is negative
   * and b is not a whole number for any size class.</li>
   * <li>Height-GLI Weibull mortality is enabled, and max mortality is not a
   * proportion.</li>
   * <li>etc...</li>
   * </ul>
   */
  public void ValidateData(TreePopulation oPop) throws ModelException {
    Behavior oBeh;
    int iNumSpecies = oPop.GetNumberOfSpecies(),
        i;

    //If no behaviors are enabled, exit
    if (!AnyBehaviorsEnabled()) {
      return;
    }

    //For each behavior, make sure that the appropriate data is good
    //GMF mortality
    oBeh = GetBehaviorByKey("gmfmort");
    if (oBeh.m_bIsEnabled) {
      MakeSureRightSize(mp_fMortAtZeroGrowth, iNumSpecies);
      MakeSureRightSize(mp_fLightDependentMortality, iNumSpecies);

      if (m_oManager.GetPlot().GetNumberOfYearsPerTimestep() != 5) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 "GMF Mortality cannot be used unless the " +
                                 "timestep length is five years."));
      }

      MakeSureAllPositive(mp_fLightDependentMortality,
                          oBeh.GetWhichSpeciesUsed(oPop));
    }

    //BC mortality
    oBeh = GetBehaviorByKey("bcmort");
    if (oBeh.m_bIsEnabled) {
      MakeSureRightSize(mp_fMortAtZeroGrowth, iNumSpecies);
      MakeSureRightSize(mp_fLightDependentMortality, iNumSpecies);
      MakeSureAllPositive(mp_fLightDependentMortality,
                          oBeh.GetWhichSpeciesUsed(oPop));
    }

    //Senescence
    oBeh = GetBehaviorByKey("senescence");
    if (oBeh.m_bIsEnabled) {
      MakeSureRightSize(mp_fDbhAtOnsetOfSenescence, iNumSpecies);
      MakeSureRightSize(mp_fRandomMortalityAlpha, iNumSpecies);
      MakeSureRightSize(mp_fRandomMortalityBeta, iNumSpecies);
      MakeSureGreaterThan(m_iDbhAtAsymptoticMaximumMortality, 0);
      MakeSureAllPositive(mp_fDbhAtOnsetOfSenescence,
                          oBeh.GetWhichSpeciesUsed(oPop));

    }

    //Adult self-thinning
    oBeh = GetBehaviorByKey("adultselfthin");
    if (oBeh.m_bIsEnabled) {
      MakeSureRightSize(mp_fAdultSelfThinningIntercept, iNumSpecies);
      MakeSureRightSize(mp_fAdultSelfThinningMaxDbh, iNumSpecies);
      MakeSureRightSize(mp_fAdultSelfThinningSlope, iNumSpecies);
      MakeSureAllPositive(mp_fAdultSelfThinningMaxDbh,
                          oBeh.GetWhichSpeciesUsed(oPop));
    }

    //Juvenile self-thinning
    oBeh = GetBehaviorByKey("juvselfthin");
    if (oBeh.m_bIsEnabled) {
      MakeSureRightSize(mp_fJuvenileSelfThinningIntercept, iNumSpecies);
      MakeSureRightSize(mp_fJuvenileSelfThinningSlope, iNumSpecies);
    }

    //Adult stochastic mortality
    oBeh = GetBehaviorByKey("adultstochmort");
    if (oBeh.m_bIsEnabled) {
      MakeSureRightSize(mp_fAdultRandomMortality, iNumSpecies);
      MakeSureAllAreProportions(mp_fAdultRandomMortality,
                                oBeh.GetWhichSpeciesUsed(oPop));
    }

    //Juvenile stochastic mortality
    oBeh = GetBehaviorByKey("juvstochmort");
    if (oBeh.m_bIsEnabled) {
      MakeSureRightSize(mp_fJuvenileRandomMortality, iNumSpecies);
      MakeSureAllAreProportions(mp_fJuvenileRandomMortality,
                                oBeh.GetWhichSpeciesUsed(oPop));
    }

    //Weibull snag mortality
    oBeh = GetBehaviorByKey("weibull snag mortality");
    if (oBeh.m_bIsEnabled) {
      boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
      //MakeSureAllNonZero(mp_fSnag1WeibullA, p_bAppliesTo);
      //MakeSureAllNonZero(mp_fSnag2WeibullA, p_bAppliesTo);
      //MakeSureAllNonZero(mp_fSnag3WeibullA, p_bAppliesTo);

      //Make sure something's entered

      int iCount = 0;
      for (i = 0; i < p_bAppliesTo.length; i++) {
        if (p_bAppliesTo[i] == true) iCount++;
      }

      if(mp_fSnag1WeibullA.GetValue().size() < iCount) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
             m_sName + ":  There is the wrong number of values in " +
          mp_fSnag1WeibullA.GetDescriptor() +
          ".  Expected: " + iCount + "; actual: " + mp_fSnag1WeibullA.GetValue().size() +
          "."));
      }
      if(mp_fSnag2WeibullA.GetValue().size() < iCount) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
             m_sName + ":  There is the wrong number of values in " +
          mp_fSnag2WeibullA.GetDescriptor() +
          ".  Expected: " + iCount + "; actual: " + mp_fSnag2WeibullA.GetValue().size() +
          "."));
      }
      if(mp_fSnag3WeibullA.GetValue().size() < iCount) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
             m_sName + ":  There is the wrong number of values in " +
          mp_fSnag3WeibullA.GetDescriptor() +
          ".  Expected: " + iCount + "; actual: " + mp_fSnag3WeibullA.GetValue().size() +
          "."));
      }
      if(mp_fSnag1WeibullB.GetValue().size() < iCount) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
             m_sName + ":  There is the wrong number of values in " +
          mp_fSnag1WeibullB.GetDescriptor() +
          ".  Expected: " + iCount + "; actual: " + mp_fSnag1WeibullB.GetValue().size() +
          "."));
      }
      if(mp_fSnag2WeibullB.GetValue().size() < iCount) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
             m_sName + ":  There is the wrong number of values in " +
          mp_fSnag2WeibullB.GetDescriptor() +
          ".  Expected: " + iCount + "; actual: " + mp_fSnag2WeibullB.GetValue().size() +
          "."));
      }
      if(mp_fSnag3WeibullB.GetValue().size() < iCount) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
             m_sName + ":  There is the wrong number of values in " +
          mp_fSnag3WeibullB.GetDescriptor() +
          ".  Expected: " + iCount + "; actual: " + mp_fSnag3WeibullB.GetValue().size() +
          "."));
      }

      //OK, if a is negative, b has to be a whole number
      Number oA, oB;
      for (i = 0; i < p_bAppliesTo.length; i++) {
        if (p_bAppliesTo[i]) {
          oA = (Number) mp_fSnag1WeibullA.GetValue().elementAt(i);
          if (oA.floatValue() < 0) {
            oB = (Number) mp_fSnag1WeibullB.GetValue().elementAt(i);
            if (oB.floatValue() <= 0 || oB.floatValue() - oB.intValue() > 0) {
              throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
              "Values for " + mp_fSnag1WeibullA.GetDescriptor() + "\n must be greater than zero if that species's value for \n"
              + mp_fSnag1WeibullB.GetDescriptor() + "\n is not a whole number."));
            }
          }
          oA = (Number) mp_fSnag2WeibullA.GetValue().elementAt(i);
          if (oA.floatValue() < 0) {
            oB = (Number) mp_fSnag2WeibullB.GetValue().elementAt(i);
            if (oB.floatValue() <= 0 || oB.floatValue() - oB.intValue() > 0) {
              throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
              "Values for " + mp_fSnag2WeibullA.GetDescriptor() + "\n must be greater than zero if that species's value for \n"
              + mp_fSnag2WeibullB.GetDescriptor() + "\n is not a whole number."));
            }
          }
          oA = (Number) mp_fSnag3WeibullA.GetValue().elementAt(i);
          if (oA.floatValue() < 0) {
            oB = (Number) mp_fSnag3WeibullB.GetValue().elementAt(i);
            if (oB.floatValue() <= 0 || oB.floatValue() - oB.intValue() > 0) {
              throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
              "Values for " + mp_fSnag3WeibullA.GetDescriptor() + "\n must be greater than zero if that species's value for \n"
              + mp_fSnag3WeibullB.GetDescriptor() + "\n is not a whole number."));
            }
          }
        }
      }

    }

    //NCI Mortality
    oBeh = GetBehaviorByKey("NCI Mortality");
    if (oBeh.m_bIsEnabled) {

      boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
      MakeSureAllNonNegative(mp_fNCIMaxCrowdingRadius, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCIMaxProbSurvival, p_bAppliesTo);
      MakeSureAllNonZero(mp_fNCISizeEffectMode, p_bAppliesTo);
      MakeSureAllNonZero(mp_fNCISizeEffectVariance, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCINeighStormEffFull, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCINeighStormEffMed, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCIStormEffectFull, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fNCIStormEffectMed, p_bAppliesTo);
      MakeSureAllNonNegative(mp_fNCIMinNeighborDBH); //this is for all species

      //Count the number of lambdas
      int iNumLambdas = 0;
      for (i = 0; i < oBeh.GetNumberOfRequiredDataObjects(); i++) {
        ModelData oData = (ModelData) oBeh.GetRequiredData(i);
        if (oData.GetDescriptor().toLowerCase().indexOf("nci lambda") > -1) {
          ModelVector oVectorTest = (ModelVector) oData;
          MakeSureRightSize(oVectorTest, iNumSpecies);
          //MakeSureAllAreProportions(oVectorTest, p_bAppliesTo);
          iNumLambdas++;
        }
      }

      if (iNumSpecies > iNumLambdas) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
        "There is the wrong number of NCI lambda neighbors."));
      }
    }

    //Competition mortality
    oBeh = GetBehaviorByKey("competition mortality");
    if (oBeh.m_bIsEnabled) {
      if (m_oManager.GetPlot().GetNumberOfYearsPerTimestep() != 1) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 oBeh.GetDescriptor() + " cannot be used " +
                                 "unless the timestep length is one year."));
      }
      //Make sure an NCI growth behavior is enabled
      GrowthBehaviors oGrowth = m_oManager.GetGrowthBehaviors();
      if (oGrowth.GetBehaviorByKey("nci").m_bIsEnabled == false &&
          oGrowth.GetBehaviorByKey("nci diam only").m_bIsEnabled == false) {
        throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                                  oBeh.GetDescriptor() +
                                  " must be used with an NCI growth behavior."));
      }
      boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop),
                p_bNCI;
      Behavior oNCI1 = oGrowth.GetBehaviorByKey("nci"),
               oNCI2 = oGrowth.GetBehaviorByKey("nci diam only");
      if (oNCI1.m_bIsEnabled && !oNCI2.m_bIsEnabled)
        p_bNCI = oNCI1.GetWhichSpeciesUsed(oPop);
      else if (!oNCI1.m_bIsEnabled && oNCI2.m_bIsEnabled)
        p_bNCI = oNCI2.GetWhichSpeciesUsed(oPop);
      else {
        p_bNCI = MergeBooleans(oNCI1.GetWhichSpeciesUsed(oPop), 
                               oNCI2.GetWhichSpeciesUsed(oPop));
      }
      for (i = 0; i < p_bAppliesTo.length; i++) 
        if (p_bAppliesTo[i] && !p_bNCI[i])
          throw (new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
              oPop.GetSpeciesNameFromCode(i) +
              " must use an NCI growth behavior in order to use the " +
              oBeh.GetDescriptor() + " behavior."));
    }

    //Density self-thinning
    oBeh = GetBehaviorByKey("density self-thinning mortality");
    boolean[] p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);

    //Make sure the timestep length is 1 year
    if (oBeh.m_bIsEnabled) {
      if (m_oManager.GetPlot().GetNumberOfYearsPerTimestep() != 1) {
        throw(new ModelException(ErrorGUI.BAD_DATA, "JAVA",
                                 oBeh.GetDescriptor() + " cannot be used " +
                                 "unless the timestep length is one year."));
      }

      MakeSureAllPositive(mp_fDensSelfThinNeighRadius, p_bAppliesTo);
      MakeSureAllNonNegative(mp_fDensSelfThinMinDensity, p_bAppliesTo);
      MakeSureAllPositive(mp_fDensSelfThinAsymptote, p_bAppliesTo);
      MakeSureAllPositive(mp_fDensSelfThinDensityEffect, p_bAppliesTo);
    }

    //Logistic bi-level
    if (GetBehaviorByKey("logistic bi-level mortality").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("logistic bi-level mortality").
          GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fLogBiLevHiLiteThreshold, p_bAppliesTo, 0, 100);
    }

    //Stochastic bi-level
    if (GetBehaviorByKey("stochastic bi-level mortality").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("stochastic bi-level mortality").
          GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fStochBiLevHiLiteThreshold, p_bAppliesTo, 0, 100);
      MakeSureAllAreProportions(mp_fStochBiLevHiLiteMortProb, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fStochBiLevLoLiteMortProb, p_bAppliesTo);
      if (m_oManager.GetLightBehaviors().GetBehaviorByKey("storm light").m_bIsEnabled == false) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", "If the \"" +
          GetBehaviorByKey("stochastic bi-level mortality").GetDescriptor() +
          "\" behavior is enabled, you must also enable the \"" +
          m_oManager.GetLightBehaviors().GetBehaviorByKey("storm light").GetDescriptor() +
          "\" behavior."));
      }
    }
     
    //Stochastic bi-level
    if (GetBehaviorByKey("stochastic bi-level mortality gli").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("stochastic bi-level mortality gli").
          GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fStochBiLevHiLiteThreshold, p_bAppliesTo, 0, 100);
      MakeSureAllAreProportions(mp_fStochBiLevHiLiteMortProb, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fStochBiLevLoLiteMortProb, p_bAppliesTo);     
    }

    //Height-GLI weibull
    if (GetBehaviorByKey("height-gli weibull mortality").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("height-gli weibull mortality").
          GetWhichSpeciesUsed(oPop);
      MakeSureAllAreProportions(mp_fHeightGLIWeibMaxMort, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fHeightGLIWeibBrowsedMaxMort, p_bAppliesTo);
    }

    //Aggregated mortality
    if (GetBehaviorByKey("aggregated mortality").m_bIsEnabled) {
      MakeSureIsProportion(m_fAggMortPropToKill);
      MakeSureGreaterThan(m_fAggMortEpisodeReturnInterval, 0);
    }

    //Browsed stochastic mortality
    if (GetBehaviorByKey("browsed stochastic mortality").m_bIsEnabled) {
      p_bAppliesTo = GetBehaviorByKey("browsed stochastic mortality").
          GetWhichSpeciesUsed(oPop);
      MakeSureAllAreProportions(mp_fJuvenileRandomMortality, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fBrowsedJuvenileRandomMortality, p_bAppliesTo);
    }
    
    //Gompertz density self-thinning
    oBeh = GetBehaviorByKey("Gompertz Density Self Thinning");
    if (oBeh.m_bIsEnabled) {  
      p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
      MakeSureAllNonNegative(mp_fGompertzMinHeight, p_bAppliesTo);
      MakeSureGreaterThan(m_fGompertzSearchRadius, 0);
    }
    
    //Weibull climate survival 
    oBeh = GetBehaviorByKey("weibull climate survival");
    if (oBeh.m_bIsEnabled) {  
      p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
      MakeSureAllAreProportions(mp_fWeibClimAdultMaxSurv, p_bAppliesTo);
      MakeSureAllAreProportions(mp_fWeibClimJuvMaxSurv, p_bAppliesTo);
      MakeSureAllNonNegative(mp_fWeibClimMaxCrowdingRadius, p_bAppliesTo);
      MakeSureAllNonZero(mp_fWeibClimAdultPrecipA, p_bAppliesTo);
      MakeSureAllNonZero(mp_fWeibClimAdultTempA, p_bAppliesTo);
      MakeSureAllNonZero(mp_fWeibClimJuvenilePrecipA, p_bAppliesTo);
      MakeSureAllNonZero(mp_fWeibClimJuvenileTempA, p_bAppliesTo);
      MakeSureAllPositive(mp_fWeibClimAdultSizeX0, p_bAppliesTo);
      MakeSureAllNonZero(mp_fWeibClimAdultSizeXb, p_bAppliesTo);
      MakeSureAllPositive(mp_fWeibClimJuvenileSizeX0, p_bAppliesTo);
      MakeSureAllNonZero(mp_fWeibClimJuvenileSizeXb, p_bAppliesTo);
    }
    
    //Temperature dependent neighborhood survival 
    oBeh = GetBehaviorByKey("temp dependent neigh surv");
    if (oBeh.m_bIsEnabled) {
      p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
      MakeSureAllNonZero(mp_fTempDepSurvN, p_bAppliesTo);
      MakeSureGreaterThanEqualTo(m_fTempDepSurvSearchRadius, 0);
    }
    
    //Insect infestation mortality
    oBeh = GetBehaviorByKey("insect mortality");
    if (oBeh.m_bIsEnabled) {
      p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fInsectMortIntercept, p_bAppliesTo, 0, 1);
      MakeSureAllAreBounded(mp_fInsectMortMax, p_bAppliesTo, 0, 1);
      MakeSureAllNonZero(mp_fInsectMortX0, p_bAppliesTo);
      //Make sure insect infestation is also enabled for all species/types
      Behavior oBeh2 = m_oManager.GetDisturbanceBehaviors().
              GetBehaviorByKey("Insect Infestation");
      if (oBeh2.m_bIsEnabled == false) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", 
            "If the \"" + oBeh.GetDescriptor() +
            "\" behavior is used, you must also use the \"" +
            oBeh2.GetDescriptor() + "\" behavior."));
      }
      boolean bFound;
      int j;
      for (i = 0; i < oBeh.GetNumberOfCombos(); i++) {
        bFound = false;
        for (j = 0; j < oBeh2.GetNumberOfCombos(); j++) {
          if (oBeh.GetSpeciesTypeCombo(i).equals(oBeh2.GetSpeciesTypeCombo(j))) {
            bFound = true;
            break;
          } 
        }
        if (bFound == false) {
          throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", 
              "The \"" + oBeh2.GetDescriptor() +
              "\" behavior must be applied to all trees to which the \"" +
              oBeh.GetDescriptor() + "\" behavior is applied."));
        }
      }
    }
    
    //Suppression duration mortality
    oBeh = GetBehaviorByKey("suppression duration mortality");
    if (oBeh.m_bIsEnabled) {
      p_bAppliesTo = oBeh.GetWhichSpeciesUsed(oPop);
      MakeSureAllAreBounded(mp_fSuppDurrMortMax, p_bAppliesTo, 0, 1);
      MakeSureAllNonZero(mp_fSuppDurrMortX0, p_bAppliesTo);
      //Make sure tree age is also enabled for all species/types
      /*Behavior oBeh2 = m_oManager.GetAnalysisBehaviors().
              GetBehaviorByKey("Tree Age");
      if (oBeh2.m_bIsEnabled == false) {
        throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", 
            "If the \"" + oBeh.GetDescriptor() +
            "\" behavior is used, you must also use the \"" +
            oBeh2.GetDescriptor() + "\" behavior."));
      }
      boolean bFound;
      int j;
      for (i = 0; i < oBeh.GetNumberOfCombos(); i++) {
        bFound = false;
        for (j = 0; j < oBeh2.GetNumberOfCombos(); j++) {
          if (oBeh.GetSpeciesTypeCombo(i).equals(oBeh2.GetSpeciesTypeCombo(j))) {
            bFound = true;
            break;
          } 
        }
        if (bFound == false) {
          throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA", 
              "The \"" + oBeh2.GetDescriptor() +
              "\" behavior must be applied to all trees to which the \"" +
              oBeh.GetDescriptor() + "\" behavior is applied."));
        }
      }*/
    }
  }

  /**
   * Finds a behavior by its XML tag.  For backwards compatibility, we'll
   * watch out for the old name of the NCI Mortality behavior for a while.
   * @param sXMLTag String The XML tag for which to find a behavior.
   * @return Behavior Behavior for the XML tag, or NULL if none of the
   * behaviors has that tag.
   */
  public Behavior GetBehaviorByXMLTag(String sXMLTag) {
    if (sXMLTag.equals("puerto rico nci mortality")) {
      return GetBehaviorByKey("NCI Mortality");
    }
    return super.GetBehaviorByXMLTag(sXMLTag);
  }

  /**
   * Updates the lambda neighbor names when a species name is changed.
   * @param sOldSpecies String Old name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)
   * @param sNewSpecies String New name of the species, with underscores
   * instead of spaces (like the species names would come from the tree
   * population)

   */
  public void ChangeOfSpeciesName (String sOldSpecies, String sNewSpecies) {
    int i;

    sOldSpecies = sOldSpecies.replace('_', ' ');

    //Find the lambdas in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      ModelData oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sOldSpecies) > -1) {
        ModelVector oVector = (ModelVector) oData;
        oVector.m_sDescriptor = sNewSpecies.replace('_', ' ') + " NCI Lambda Neighbors";
        oVector.m_sXMLTag = "mo_prnci" + sNewSpecies + "NeighborLambda";
      }
    }
  }

  /**
   * Updates the lambda when a species is copied. The lambdas for that neighbor
   * are made identical to those being copied as well as entries for species
   * within the lambda.
   * @param iSpeciesCopyFrom int Species to copy.
   * @param iSpeciesCopyTo int Species that is the copy.
   * @throws ModelException if there is a problem.
   */
  public void CopySpecies (int iSpeciesCopyFrom, int iSpeciesCopyTo) throws ModelException {
    TreePopulation oPop = m_oManager.GetTreePopulation();
    ModelVector oCopyFrom = null, oCopyTo = null;
    ModelData oData;
    Float f1, f2;
    String sCopyFrom = oPop.GetSpeciesNameFromCode(iSpeciesCopyFrom).replace('_', ' '),
           sCopyTo = oPop.GetSpeciesNameFromCode(iSpeciesCopyTo).replace('_', ' ');
    int i;
    //Names can be very short - like "A" - so make sure that they are 
    //surrounded by whitespace to ensure they were a whole word
    sCopyFrom = sCopyFrom.concat(" ");
    sCopyTo = sCopyTo.concat(" ");

    //Find the lambdas in our list
    for (i = 0; i < mp_oAllData.size(); i++) {
      oData = (ModelData) mp_oAllData.elementAt(i);
      String sKey = oData.GetDescriptor();
      if (sKey.indexOf(sCopyFrom) > -1 &&
          sKey.indexOf("Lambda") > -1) {
        oCopyFrom = (ModelVector) oData;
      }
      if (sKey.indexOf(sCopyTo) > -1 &&
          sKey.indexOf("Lambda") > -1) {
        oCopyTo = (ModelVector) oData;
      }
    }

    if (null == oCopyFrom || null == oCopyTo) {
      throw(new ModelException(ErrorGUI.CANT_FIND_OBJECT, "JAVA",
                               "Mortality could not find NCI lambdas for neighbor"
                               + " species when copying species."));
    }

    if (oCopyTo.GetValue().size() > 0) {
      for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().remove(i);
        oCopyTo.GetValue().add(i, f2);
      }
    } else {
    	for (i = 0; i < oCopyFrom.GetValue().size(); i++) {
        f1 = (Float)oCopyFrom.GetValue().elementAt(i);
        if (null == f1)
          f2 = null;
        else
          f2 = new Float(f1.floatValue());
        oCopyTo.GetValue().add(i, f2);
      }
    }

    super.CopySpecies(iSpeciesCopyFrom, iSpeciesCopyTo);
  }
}
